from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template

# this hooks up the example html file
urlpatterns = patterns('',
                       url(r'',
                           direct_to_template, {'template':'stwexample.html'},
                           ),
                       )



"""Tests for custom templatetags"""
import unittest
from mock import Mock, patch
from django.conf import settings
from django import template
from stw.templatetags.shrinkthewebtags import (FormatSTWFreeImageNode,
                                               FormatSTWImageNode,
                                               do_shrinkthewebimage,
                                               do_stwimage,
                                               STWConfigError,
                                               stwjavascript)

class TestFormatSTWFreeImageNode(unittest.TestCase):

    def setUp(self):
        self.settings = settings.SHRINK_THE_WEB
        settings.SHRINK_THE_WEB = {'stwaccesskeyid' : 'key'}

    def tearDown(self):
        settings.SHRINK_THE_WEB = self.settings

    def test_render_strings_url(self):
        # ignore kwarg URL params
        node = FormatSTWFreeImageNode("'url'", **{'stwsize':'sm'})
        def side_effect(*args, **kwargs):
            return 'url'
        node._resolve = Mock(side_effect=side_effect)

        self.assertEqual("""<script type="text/javascript">stw_pagepix('url','key','sm','en');</script>""", node.render(None))

    def test_render_strings_url_lang_default(self):
        settings.SHRINK_THE_WEB = {'stwaccesskeyid': 'key',
                                   'lang': 'fr'}
        node = FormatSTWFreeImageNode("'url'", **{'stwsize':'sm'})
        def side_effect(*args, **kwargs):
            return 'url'
        node._resolve = Mock(side_effect=side_effect)

        self.assertEqual("""<script type="text/javascript">stw_pagepix('url','key','sm','fr');</script>""", node.render(None))


    def test_render_strings_url_lang_local(self):
        settings.SHRINK_THE_WEB = {'stwaccesskeyid': 'key',
                                   'lang': 'fr'}
        node = FormatSTWFreeImageNode("'url'", **{'stwsize':'sm', 'lang':'de'})
        def side_effect(*args, **kwargs):
            return 'url'
        node._resolve = Mock(side_effect=side_effect)

        self.assertEqual("""<script type="text/javascript">stw_pagepix('url','key','sm','de');</script>""", node.render(None))

    def test_render_strings_url_one_option(self):
        node = FormatSTWFreeImageNode("'url'", stwsize='lrg', stwfull=1)
        def side_effect(*args, **kwargs):
            return 'url'
        node._resolve = Mock(side_effect=side_effect)
        self.assertEqual("""<script type="text/javascript">stw_pagepix('url','key','lrg','en','stwfull=1');</script>""", node.render(None))

    def test_render_strings_url_two_options(self):
        node = FormatSTWFreeImageNode("'url'", stwsize='lrg', stwfull=1, stwsomearg='abcde')
        def side_effect(*args, **kwargs):
            return 'url'
        node._resolve = Mock(side_effect=side_effect)
        self.assertEqual("""<script type="text/javascript">stw_pagepix('url','key','lrg','en','stwfull=1&stwsomearg=abcde');</script>""", node.render(None))


class TestSTWImageNode(unittest.TestCase):

    def setUp(self):
        self.settings = settings.SHRINK_THE_WEB
        settings.SHRINK_THE_WEB = {'stwaccesskeyid' : 'key'}

    def tearDown(self):
        settings.SHRINK_THE_WEB = self.settings

    def test_init(self):
        node = FormatSTWImageNode("url", "alt", stwsize='lrg')
        self.assertEqual("url", "%s" % node.url)
        self.assertEqual("alt", node.alt)
        # get value from settings.SHRINK_THE_WEB
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])

    def test_init_override_key(self):
        node = FormatSTWImageNode("url", "alt", stwaccesskeyid='overridekey', stwsize='lrg')
        self.assertEqual("url", "%s" % node.url)
        self.assertEqual("alt", node.alt)
        self.assertEqual('overridekey', node.kwargs['stwaccesskeyid'])

    def test_init_add_from_settings_and_override_key(self):
        settings.SHRINK_THE_WEB = {'stwaccesskeyid' : 'key', 'stwanewkey': 'newkey'}
        node = FormatSTWImageNode("url", "alt", stwaccesskeyid='overridekey', stwsize='lrg')
        self.assertEqual("url", "%s" % node.url)
        self.assertEqual("alt", node.alt)
        self.assertEqual('overridekey', node.kwargs['stwaccesskeyid'])
        self.assertEqual('newkey', node.kwargs['stwanewkey'])


    # valid combinations of size, xmax, ymax
    def test_init_no_stwembed(self):
        node = FormatSTWImageNode('url', 'alt', **{'stwaccesskeyid':'key', 'stwsize':'lrg'})
        self.assertEqual(3, len(node.kwargs.keys()))
        self.assertEqual(1, node.kwargs['stwembed'])
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])
        self.assertEqual('lrg', node.kwargs['stwsize'])

    def test_init_stwembed(self):
        node = FormatSTWImageNode('url', 'alt', **{'stwaccesskeyid':'key', 'stwembed':0, 'stwsize':'lrg'})
        self.assertEqual(3, len(node.kwargs.keys()))
        self.assertEqual(0, node.kwargs['stwembed'])
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])
        self.assertEqual('lrg', node.kwargs['stwsize'])

    def test_init_stwembed_stwxmax(self):
        node = FormatSTWImageNode('url', 'alt', **{'stwaccesskeyid':'key', 'stwembed':0, 'stwxmax':100})
        self.assertEqual(3, len(node.kwargs.keys()))
        self.assertEqual(0, node.kwargs['stwembed'])
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])
        self.assertEqual(100, node.kwargs['stwxmax'])

    def test_init_stwembed_stwymax(self):
        node = FormatSTWImageNode('url', 'alt', **{'stwaccesskeyid':'key', 'stwembed':0, 'stwymax':100})
        self.assertEqual(3, len(node.kwargs.keys()))
        self.assertEqual(0, node.kwargs['stwembed'])
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])
        self.assertEqual(100, node.kwargs['stwymax'])

    def test_init_stwembed_stwxmax_stwymax(self):
        node = FormatSTWImageNode('url', 'alt', **{'stwaccesskeyid':'key', 'stwembed':0, 'stwymax':200, 'stwxmax':100})
        self.assertEqual(4, len(node.kwargs.keys()))
        self.assertEqual(0, node.kwargs['stwembed'])
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])
        self.assertEqual(200, node.kwargs['stwymax'])
        self.assertEqual(100, node.kwargs['stwxmax'])

    def test_init_stwembed_stwxfull(self):
        node = FormatSTWImageNode('url', 'alt', **{'stwaccesskeyid':'key', 'stwembed':0, 'stwfull':1})
        self.assertEqual(3, len(node.kwargs.keys()))
        self.assertEqual(0, node.kwargs['stwembed'])
        self.assertEqual('key', node.kwargs['stwaccesskeyid'])
        self.assertEqual(1, node.kwargs['stwfull'])

    # missing required configuration
    def test_init_no_stwaccesskeyid(self):
        settings.SHRINK_THE_WEB = {}
        self.assertRaises(template.TemplateSyntaxError,
                          FormatSTWImageNode,
                          'url', 'alt', stwsize='lrg')

    # invalid combinations of stwsize, stwxmax, stwymax and stwfull
    def test_init_no_stwsize_no_stwxmax_no_stwymax(self):
        self.assertRaises(template.TemplateSyntaxError, FormatSTWImageNode,
                          'url', 'alt', stwaccesskeyid='key')

    def test_init_stwsize_stwfull(self):
        self.assertRaises(template.TemplateSyntaxError, FormatSTWImageNode,
                          'url', 'alt', **{'stwaccesskeyid':'key', 'stwsize':'lrg', 'stwfull':1})

    def test_init_stwsize_stwxmax(self):
        self.assertRaises(template.TemplateSyntaxError, FormatSTWImageNode,
                          'url', 'alt', **{'stwaccesskeyid':'key', 'stwsize':'lrg', 'stwxmax':100})

    def test_init_stwsize_stwymax(self):
        self.assertRaises(template.TemplateSyntaxError, FormatSTWImageNode,
                          'url', 'alt', **{'stwaccesskeyid':'key', 'stwsize':'lrg', 'stwymax':100})

    def test_init_stwsize_stwxmax_stwymax(self):
        self.assertRaises(template.TemplateSyntaxError, FormatSTWImageNode,
                          'url', 'alt', **{'stwaccesskeyid':'key', 'stwsize':'lrg', 'stwymax':100, 'stwxmax':100})

    def test_render_context(self):
        node = FormatSTWImageNode("url", "alt", stwsize='lrg')
        results = ["alt", "url"]
        def side_effect(*args, **kwargs):
            return results.pop()
        node._resolve = Mock(side_effect=side_effect)

        context = {'alt': 'contextalt'}
        self.assertEqual('''<img src="http://images.shrinktheweb.com/xino.php?stwaccesskeyid=key&stwsize=lrg&stwembed=1&stwurl=url" alt="alt"/>''', node.render(context))

    @patch('urllib.urlencode')
    def test_render_strings_url_alt_kwargs(self, mockurlencode):
        node = FormatSTWImageNode("'url'", "'alt'", stwsize='lrg')
        results = ["alt", "url"]
        def side_effect(*args, **kwargs):
            return results.pop()
        node._resolve = Mock(side_effect=side_effect)
        mockurlencode.return_value = "kwarg=kwargvalue"
        self.assertEqual('''<img src="http://images.shrinktheweb.com/xino.php?kwarg=kwargvalue&stwurl=url" alt="alt"/>''', node.render(None))


class TestDoShrinkTheWebImage(unittest.TestCase):

    def test_no_args(self):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=('tagname',))
        self.assertRaises(template.TemplateSyntaxError, do_shrinkthewebimage, parser, token)

    @patch('stw.templatetags.shrinkthewebtags.FormatSTWFreeImageNode')
    def test_correct_args(self, MockClass):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=("tagname", "url", "lrg"))
        do_shrinkthewebimage(parser, token)
        MockClass.assert_called_with(url="url", stwsize="lrg")

    @patch('stw.templatetags.shrinkthewebtags.FormatSTWFreeImageNode')
    def test_pro_args(self, MockClass):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=("tagname", "url", "lrg", 'stwinside = 1', 'stwanotherkey = asfd'))
        do_shrinkthewebimage(parser, token)
        MockClass.assert_called_with(url="url", stwsize="lrg", stwinside="1", stwanotherkey='asfd')


class TestDoSTWImage(unittest.TestCase):

    def test_no_args(self):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=('tagname',))
        self.assertRaises(template.TemplateSyntaxError, do_stwimage, parser, token)

    @patch('stw.templatetags.shrinkthewebtags.FormatSTWImageNode')
    def test_no_kwargs(self, MockClass):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=("stwimage", "url", "alt"))
        do_stwimage(parser, token)
        MockClass.assert_called_with(url="url", alt="alt")

    @patch('stw.templatetags.shrinkthewebtags.FormatSTWImageNode')
    def test_stw_kwarg(self, MockClass):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=("stwimage", "url", "alt", "stwsize=lrg"))
        do_stwimage(parser, token)
        MockClass.assert_called_with(url="url", alt="alt", stwsize="lrg")

    @patch('stw.templatetags.shrinkthewebtags.FormatSTWImageNode')
    def test_stw_kwargs(self, MockClass):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=("stwimage", "url", "alt", "stwsize=lrg", "stwembed=1"))
        do_stwimage(parser, token)
        MockClass.assert_called_with(url="url", alt="alt", stwsize="lrg", stwembed='1')

    def test_nonstw_kwarg(self):
        parser = Mock()
        token = Mock()
        token.split_contents = Mock(return_value=("stwimage", "url", "alt", "size=lrg"))
        self.assertRaises(template.TemplateSyntaxError, do_stwimage, parser, token)

class TestSTWJavaScript(unittest.TestCase):

    def test_is_script(self):
        string = stwjavascript()
        self.assertTrue("<script " in string and "</script>" in string)

    def test_correct_url(self):
        string = stwjavascript()
        self.assertTrue("http://www.shrinktheweb.com/scripts/pagepix.js" in string)



from setuptools import setup, find_packages

setup(
    name='django-stw',
    version='0.3.0',
    description='This application provides templatetags for simplying using Shrink The Web PagePix',
    author='Steve Schwarz',
    author_email='steve@agilitynerd.com',
    url='http://github.com/saschwarz/django-stw',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=['setuptools',],
)



"""
Django template tags for inserting Shrink The Web images into templates.

There are two templatetags:
 - shrinkthewebimage - the original image insertion templatetag that implements
the STW preview feature. STW requires free accounts to use this tag. PRO accounts
can use this tag to access the preview feature.
 - stwimage - the non-preview templatetag that supports all PRO features.
"""
import urllib
from django.conf import settings
from django import template

class STWConfigError(template.TemplateSyntaxError):
    pass

class FormatSTWFreeImageNode(template.Node):

    def __init__(self, url, **kwargs):
        params = {}
        # load defaults if any
        params.update(settings.SHRINK_THE_WEB)
        # overwrite defaults for this tag instance
        params.update(kwargs)
        self.kwargs = params
        self.url = url
        self._validate()

    def _validate(self):
        if 'stwaccesskeyid' not in self.kwargs:
            raise STWConfigError("'stwaccesskeyid' must be defined in settings.SHRINK_THE_WEB")
        if 'stwsize' not in self.kwargs:
            raise template.TemplateSyntaxError("'shrinkthewebimage' tag requires 'stwsize' keyword")

    @classmethod
    def _resolve(cls, var, context):
        """if var is a string then return it otherwise use it to lookup a value in the current context"""
        if var[0] == var[-1] and var[0] in ('"', "'"):
            var = var[1:-1] # a string
        else:
            var = template.Variable(var).resolve(context)
        return var

    def _create_urlencoded_options(self, options):
        """Create JSON dictionary of option excluding 'stwaccesskeyid', 'stwlang' and 'stwsize'"""
        ret = {}
        for k,v in options.items():
            if k not in ('stwaccesskeyid', 'stwsize', 'lang'):
                ret[k] = v
        return ret and urllib.urlencode(ret) or None

    def render(self, context):
        args = [self._resolve(self.url, context),
                self.kwargs['stwaccesskeyid'],
                self.kwargs['stwsize'],
                self.kwargs.get('lang', 'en')]
        options = self._create_urlencoded_options(self.kwargs)
        if options:
            args.append(options)
        return """<script type="text/javascript">stw_pagepix(%s);</script>""" % ",".join(["\'%s\'" % arg for arg in args])


class FormatSTWImageNode(FormatSTWFreeImageNode):

    def __init__(self, url, alt, **kwargs):
        if 'stwembed' not in kwargs:
            kwargs['stwembed'] = 1 # default to image
        self.alt = alt
        super(FormatSTWImageNode, self).__init__(url, **kwargs)

    def _validate(self):
        if 'stwaccesskeyid' not in self.kwargs:
            raise STWConfigError("'stwaccesskeyid' must be defined in settings.SHRINK_THE_WEB")
        # validate conflicting options
        if 'stwsize' in self.kwargs:
            if 'stwxmax' in self.kwargs or 'stwymax' in self.kwargs or 'stwfull' in self.kwargs:
                raise template.TemplateSyntaxError("'stwimage' tag does not allow 'stwsize' and ('stwfull' or ('stwxmax' and/or 'stwymax')) keyword(s)")
        elif 'stwxmax' not in self.kwargs and 'stwymax' not in self.kwargs and 'stwfull' not in self.kwargs:
            raise template.TemplateSyntaxError("'stwimage' tag requires 'stwsize' or ('stwfull' or ('stwxmax' and/or 'stwymax')) keyword(s)")

    def render(self, context):
        url = self._resolve(self.url, context)
        alt = self._resolve(self.alt, context)
        encoded = urllib.urlencode(self.kwargs)
        if encoded:
            encoded += '&'
        result =  '''<img src="http://images.shrinktheweb.com/xino.php?%sstwurl=%s" alt="%s"/>''' % (encoded, url, alt)
        return result


def do_shrinkthewebimage(parser, token):
    """
    Original templatetag designed for use by free accounts and using STW's
    verification JavaScript API. PRO account key-value options are also
    supported

    Usage::

        {% load shrinkthewebtags %}
        {% stwjavascript %}

        {% shrinkthewebimage url size key-value-pairs %}

    Where:

        ``url``
          is expected to be a variable instantiated from the context
          or a quoted string to be used explicitly.

        ``size``
          is expected to a valid size string accepted by shrinktheweb.com
          which currently accepts one of: "mcr", "tny", "vsm", "sm", "lg"
          or "xlg". A context variable can also be used.

        ``key-value-pairs``
          matching STW API values i.e. stwembed=0 stwinside=1
          minimal validation of key value pairs is performed.
          Only STW PRO accounts can supply these options.

    Examples::

        Given a template context variable "author" with attributes "url" and
        "description" the following are valid entries in a template file:

        {% shrinkthewebimage author.url "xlg"%}

        {% shrinkthewebimage author.url "xlg" stwinside=1 %}

    """
    bits = token.split_contents()
    if len(bits) < 3:
        raise template.TemplateSyntaxError("'%s' tag takes 3 or more arguments" % bits[0])
    size = bits[2]
    if size[0] == size[-1] and size[0] in ('"', "'"):
        size = size[1:-1] # a string

    kwargs = {'stwsize' : size,
              }
    for key_equal_value in bits[3:]:
        key, value = key_equal_value.split("=")
        kwargs[str(key.strip())] = value.strip()

    return FormatSTWFreeImageNode(url=bits[1], **kwargs)


def do_stwimage(parser, token):
    """
    Key value based templatetag supporting all STW features for PRO and Plus users.

    Usage::

        {% load shrinkthewebtags %}
        {% stwimage url alt key-value-pairs %}

    Where:

        ``url``
          is expected to be a variable instantiated from the context
          or a quoted string to be used explicitly.

        ``key-value-pairs``
          matching STW API values i.e. stwembed=0 stwinside=1
          minimal validation of key value pairs is performed

    Examples::

        Given a template context variable "author" with attributes "url" and
        "description" the following are valid entries in a template file:

        {% load shrinkthewebtags %}

        get image of the follow the full url (not just the top level page), wait
        5 seconds, and return image in large size (this requires license with PRO
        features:

        {% stwimage author.url author.description stwinside=1 stwdelay=5 stwsize=lrg %}
    """
    bits = token.split_contents()
    if len(bits) < 3:
        raise template.TemplateSyntaxError("'%s' tag takes at least 2 arguments" % bits[0])

    # process keyword args
    kwargs = {}
    for bit in bits[3:]:
        key, value = bit.split("=")
        if value is '':
            raise template.TemplateSyntaxError("'%s' tag keyword: %s has no argument" % (bits[0], key))

        if key.startswith('stw'):
            kwargs[str(key)] = value
        else:
            raise template.TemplateSyntaxError("'%s' tag keyword: %s is not a valid STW keyword" % (bits[0], key))
    return FormatSTWImageNode(url=bits[1], alt=bits[2] , **kwargs)


def stwjavascript():
    """Insert script tag to load the javascript required by Shrink The Web's API"""
    return """<script type="text/javascript" src="http://www.shrinktheweb.com/scripts/pagepix.js"></script>"""


register = template.Library()
register.tag('shrinkthewebimage', do_shrinkthewebimage)
register.tag('stwimage', do_stwimage)
register.simple_tag(stwjavascript)
